package task1;

import java.util.LinkedList;
import java.util.Queue;

public class BinaryTree {

  /**
   * Given a binary tree, determine if it is a valid binary search tree (BST).
   *
   * @param input An array containing the level order traversal of a binary tree.
   *              Pre: input != null
   * @return True if the given binary tree is a valid BST and false otherwise.
   */
  public static boolean isValidBst(Integer[] input) {
    Node root = constructTree(input);
    return isValidBst(root);
  }

  // Return true if the given binary tree is a valid BST and false otherwise.
  private static boolean isValidBst(Node root) {
    if (root == null || isLeaf(root)) {
      return true;
    }
    if (!holdBstProperty(root)) {
      return false;
    }
    return isValidBst(root.right) && isValidBst(root.left);
  }

  // Pre: node != null
  private static boolean holdBstProperty(Node node) {
    return (node.left == null || node.val > node.left.val)
        && (node.right == null || node.val < node.right.val);
  }

  // Pre: node != null
  private static boolean isLeaf(Node node) {
    return node.left == null && node.right == null;
  }

  // Pre: input != null
  private static Node constructTree(Integer[] input) {
    int index = 0;
    Integer next = input[index++];
    Queue<Node> nodeQueue = new LinkedList<>();
    Node root = makeNode(next, nodeQueue);
    while (!nodeQueue.isEmpty() && index < input.length) {
      Node currentNode = nodeQueue.poll();
      next = input[index++];
      currentNode.left = makeNode(next, nodeQueue);
      if (index < input.length) {
        next = input[index++];
        currentNode.right = makeNode(next, nodeQueue);
      }
    }
    return root;
  }

  // Pre: queue != null
  private static Node makeNode(Integer val, Queue<Node> queue) {
    if (val != null) {
      Node node = new Node(val);
      queue.add(node);
      return node;
    }
    return null;
  }

  private static class Node {
    int val;
    Node left;
    Node right;

    Node(int val) {
      this.val = val;
    }
  }
}
