package task3;

import exceptions.EmptyException;
import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Test;

import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertFalse;
import static org.junit.jupiter.api.Assertions.assertTrue;
import static org.junit.jupiter.api.Assertions.fail;

/**
 * Testing implementations of the PriorityQueue interface.
 */
public abstract class PriorityQueueTest {
  private PriorityQueue<Integer> unit;

  @BeforeEach
  public void setupTests() {
    unit = this.createUnit();
  }

  protected abstract PriorityQueue<Integer> createUnit();

  @Test
  @DisplayName("PQ empty after construction")
  public void newQueueEmpty() {
    assertTrue(unit.empty());
  }

  @Test
  @DisplayName("best() on empty PQ throws EmptyException")
  public void newQueueNoBest() {
    try {
      unit.best();
      fail("Exception was not thrown where it was expected");
    } catch (EmptyException ex) {
      return;
    }
  }

  @Test
  @DisplayName("worst() on empty PQ throws EmptyException")
  public void newQueueNoWorst() {
    try {
      unit.worst();
      fail("Exception was not thrown where it was expected");
    } catch (EmptyException ex) {
      return;
    }
  }

  @Test
  @DisplayName("removeBest() on empty PQ throws EmptyException")
  public void newQueueNoRemoveBest() {
    try {
      unit.removeBest();
      fail("Exception was not thrown where it was expected");
    } catch (EmptyException ex) {
      return;
    }
  }

  @Test
  @DisplayName("removeWorst() on empty PQ throws EmptyException")
  public void newQueueNoRemoveWorst() {
    try {
      unit.removeWorst();
      fail("Exception was not thrown where it was expected");
    } catch (EmptyException ex) {
      return;
    }
  }

  @Test
  @DisplayName("PQ not empty after insertion")
  public void insertNotEmpty() {
    unit.insert(1);
    assertFalse(unit.empty());
  }

  @Test
  @DisplayName("PQ best() works: default Comparator")
  public void insertBestInOrder() {
    unit.insert(1);
    assertEquals(1, unit.best().intValue());

    unit.insert(3);
    assertEquals(3, unit.best().intValue());

    unit.insert(2);
    assertEquals(3, unit.best().intValue());
  }

  @Test
  @DisplayName("PQ worst() works: default Comparator")
  public void insertWorstInOrder() {
    unit.insert(3);
    assertEquals(3, unit.worst().intValue());

    unit.insert(1);
    assertEquals(1, unit.worst().intValue());

    unit.insert(2);
    assertEquals(1, unit.worst().intValue());
  }

  @Test
  @DisplayName("PQ removeBest() works: default Comparator")
  public void insertRemoveBestInOrder() {
    int[] nums = {5, 4, 3, 2, 1, 0};
    for (int i = nums.length - 1; i >= 0; i--) {
      unit.insert(nums[i]);
    }

    for (int n : nums) {
      assertEquals(n, unit.best().intValue());
      unit.removeBest();
    }
  }

  @Test
  @DisplayName("PQ removeWorst() works: default Comparator")
  public void insertRemoveWorstInOrder() {
    int[] nums = {0, 1, 2, 3, 4, 5};
    for (int i = 0; i < nums.length; i++) {
      unit.insert(nums[i]);
    }

    for (int n : nums) {
      assertEquals(n, unit.worst().intValue());
      unit.removeWorst();
    }
  }
}
